# -*- coding: utf-8 -*-

import json

from vaporize.core import convert_datetime, get_url, handle_request, query
from vaporize.utils import DotDict


class Pointer(DotDict):
    """A CloudDNS Reverse DNS Pointer (PTR)."""
    pass



